#!/usr/bin/perl -w
package POS::CheckFile;

use strict;


require Exporter;
our @ISA = qw(Exporter);


our $VERSION = 0.1;


our @EXPORT = qw(FILE TIMEOUT FILETIME GLOBALTIMEOUT DIE check check_no_sleep );

##############################
# module to check if file
# change time (ctime) was
# changed
#

#constants

my $DEFAULT_TIMEOUT        = 60;
my $DEFAULT_GLOBAL_TIMEOUT = 0;

# constructor

use File::stat;

sub new
{
    my $package = shift;
    my $data = { 'FILE' => undef, 'TIMEOUT' => undef, 'GLOBALTIMEOUT' => undef, 'DIE' => 1, };
    while (@_) {
        my $field = shift;
        if (@_) {
            $data->{$field} = shift;
        }
    }
    if (!$data->{'FILE'}) {
        return undef;
    }
    if (!$data->{'TIMEOUT'}) {
        $data->{'TIMEOUT'} = $DEFAULT_TIMEOUT;
    }
    if (!$data->{'GLOBALTIMEOUT'}) {
        $data->{'GLOBALTIMEOUT'} = $DEFAULT_GLOBAL_TIMEOUT;
    }

    my $filedata;
    if ($data->{'DIE'}) {
        $filedata = stat($data->{'FILE'}) or die "Can't stat file: $!";
        $data->{'FILETIME'} = $filedata->ctime;
    }
    else {
        $filedata = stat($data->{'FILE'});
        if (defined($filedata)) {
            $data->{'FILETIME'} = $filedata->ctime;
        }
        else {
            $data->{'FILETIME'} = 0;
        }
    }

    return bless($data);
} ## end sub new


sub check
{
    my ($this)  = @_;
    my $gtime   = 0;
    my $timesav = time();
    $this->{'GLOBALTIMEOUT'} ? ($gtime = $this->{'GLOBALTIMEOUT'}) : ($gtime = 0);

    while (1) {
        #     print $gtime . "\n";
        #     print  $this->{'GLOBALTIMEOUT'} . "\n";
        if ($gtime) {
            if ($gtime <= (time() - $timesav)) {
                #	      print "Global Timeout . \n";
                return 0;
            }
        }
        my $filechanged = $this->check_no_sleep();
        if ($filechanged) {
            return $filechanged;
        }
        #    print "sleeping for " . $this->{'TIMEOUT'} . "\n";
        sleep($this->{'TIMEOUT'});
    }
    return 0;
} ## end sub check


sub check_no_sleep
{
    my ($this) = @_;

    my $filedata;
    if ($this->{'DIE'}) {
        $filedata = stat($this->{'FILE'}) or die "Can\'t stat file: $!";
    }
    else {
        $filedata = stat($this->{'FILE'});
        if (!defined($filedata)) {
            # file does not exist
            return 0;
        }
    }

    if ($this->{'FILETIME'} != $filedata->ctime) {
        $this->{'FILETIME'} = $filedata->ctime;
        return 1;
    }
    return 0;
} ## end sub check_no_sleep


1;


