#!/usr/bin/perl -w

package POS::Studio;

use strict;

use LWP::UserAgent;
use HTTP::Request::Common qw(:DEFAULT DELETE);
use XML::Simple;
use Data::Dumper;

# version  of the package
our $VERSION = 0.1;

sub new
{
    my $package = shift;
    my $self    = {
        #init data
        'HOST' => undef,
        'USER'  => undef,
        'PASS'  => undef,
    };

    %$self = (%$self, @_);
    
    return unless defined $self->{'HOST'};
    return unless defined $self->{'USER'};
    return unless defined $self->{'PASS'};
    
    $self->{'UA'} = new LWP::UserAgent(keep_alive=>1);
    
    $self->{'API'} = 'http://' . $self->{'USER'} . ':' . $self->{'PASS'} . '@' . $self->{'HOST'};
    
    $self->{'API_VERSION'} = get_api_version($self);
    return unless defined $self->{'API_VERSION'};

    return bless($self);
} ## end sub new

sub get_api_version
{
    my ($self) = @_;

    my $res_obj = $self->{'UA'}->request(GET $self->{'API'} . "/api/v2/user/api_version");

    my $res = XML::Simple->new->XMLin($res_obj->content);
    return $res;
}


sub get_template_id
{
    my ($self, $template) = @_;

    my $res_obj = $self->{'UA'}->request(GET $self->{'API'} . "/api/v2/user/template_sets/default");

    my $res = XML::Simple->new->XMLin($res_obj->content);
    return $res->{'template'}{$template}{'appliance_id'};
}


sub get_repo_id
{
    my ($self, $repo) = @_;

    my $res_obj = $self->{'UA'}->request(GET $self->{'API'} . "/api/v2/user/repositories");

    my $res = XML::Simple->new->XMLin($res_obj->content);

    foreach my $r (keys %{$res->{'repository'}}) {
        if ($r =~ /$repo/) {
            return $res->{'repository'}{$r}{'id'};
        }
    }
}

sub get_app_id
{
    my ($self, $app) = @_;

    my $res_obj = $self->{'UA'}->request(GET $self->{'API'} . "/api/v2/user/appliances");

    my $res = XML::Simple->new->XMLin($res_obj->content, ForceArray => 1);

    foreach my $a (@{$res->{'appliance'}}) {
        if ($a->{'name'}[0] eq $app) {
            return $a->{'id'}[0];
        }
    }
    return undef;
}

sub clone_appliance
{
    my ($self, $id, $name, $arch) = @_;

    my $url = URI->new($self->{'API'} . "/api/v2/user/appliances");
    $url->query_form(clone_from => $id,
                     name => $name,
                     arch => $arch); 
    
    my $res_obj = $self->{'UA'}->request(POST $url);
    my $res = XML::Simple->new->XMLin($res_obj->content);

    return $res->{'id'};
}

sub delete_appliance
{
    my ($self, $id) = @_;

    my $url = URI->new($self->{'API'} . "/api/v2/user/appliances/$id");
    
    my $res_obj = $self->{'UA'}->request(DELETE $url);

    return 1;
}

sub add_repo
{
    my ($self, $app_id, $repo_id) = @_;
    
    my $url = URI->new($self->{'API'} . "/api/v2/user/appliances/$app_id/cmd/add_repository");
    $url->query_form(repo_id => $repo_id); 

    my $res_obj = $self->{'UA'}->request(POST $url);
    my $res = XML::Simple->new->XMLin($res_obj->content);
}

sub remove_repo
{
    my ($self, $app_id, $repo_id) = @_;
    
    my $url = URI->new($self->{'API'} . "/api/v2/user/appliances/$app_id/cmd/remove_repository");
    $url->query_form(repo_id => $repo_id); 

    my $res_obj = $self->{'UA'}->request(POST $url);
    my $res = XML::Simple->new->XMLin($res_obj->content);
}

sub add_pattern
{
    my ($self, $app_id, $pattern) = @_;

    my $url = URI->new($self->{'API'} . "/api/v2/user/appliances/$app_id/cmd/add_pattern");
    $url->query_form(name => $pattern); 
    
    my $res_obj = $self->{'UA'}->request(POST $url);
    my $res = XML::Simple->new->XMLin($res_obj->content);

}

sub add_package
{
    my ($self, $app_id, $package, $bootinclude) = @_;
    
    $bootinclude = $bootinclude ? "true" : "false";

    my $url = URI->new($self->{'API'} . "/api/v2/user/appliances/$app_id/cmd/add_package");
    $url->query_form(name => $package,
                     bootinclude => $bootinclude); 
    
    my $res_obj = $self->{'UA'}->request(POST $url);
    my $res = XML::Simple->new->XMLin($res_obj->content);

}

sub add_configuration
{
    my ($self, $app_id, $conf) = @_;
    
    my $res_obj = $self->{'UA'}->request(PUT $self->{'API'} . "/api/v2/user/appliances/$app_id/configuration",  Content => $conf );
#    my $res = XML::Simple->new->XMLin($res_obj->content);

}

sub add_background
{
    my ($self, $app_id, $file) = @_;

    my $req = POST($self->{'API'} . "/api/v2/user/appliances/$app_id/configuration/background",  Content_Type => 'form-data', Content => [ file => [ $file ]]);
    $req->method('PUT');
    my $res_obj = $self->{'UA'}->request($req);
    my $res = XML::Simple->new->XMLin($res_obj->content);
#    print Dumper($res);

}

sub add_logo
{
    my ($self, $app_id, $file) = @_;
    
    my $req = POST($self->{'API'} . "/api/v2/user/appliances/$app_id/configuration/background",  Content_Type => 'form-data', Content => [ file => [ $file ]]);
    $req->method('PUT');
    my $res_obj = $self->{'UA'}->request($req);
    my $res = XML::Simple->new->XMLin($res_obj->content);
#    print Dumper($res);

}

sub add_build_script
{
    my ($self, $app_id, $script) = @_;
    
    my $conf = XML::Simple->new->XMLout({
        configuration => [{
            scripts => [{
                build => [{
                    enabled => ['true'],
                    script => [$script]
                }]
            }]
        }]
    }, KeepRoot => 1);

    return add_configuration($self, $app_id, $conf);
}

sub add_build_script_local
{
    my ($self, $app_id, $file) = @_;
    
    open FILE, $file or die "Couldn't open file: $!"; 
    my $string = join("", <FILE>); 
    close FILE;
    return add_build_script($self, $app_id, $string);
}

sub get_builds
{
    my ($self, $app_id) = @_;
    
    my $url = URI->new($self->{'API'} . "/api/v2/user/builds");
    $url->query_form(appliance_id => $app_id); 
    
    my $res_obj = $self->{'UA'}->request(GET $url);
    my $res = XML::Simple->new->XMLin($res_obj->content);

    print Dumper($res_obj);

}

sub get_running_builds
{
    my ($self, $app_id) = @_;
    my $url = URI->new($self->{'API'} . "/api/v2/user/running_builds");
    $url->query_form(appliance_id => $app_id); 
    my $res_obj = $self->{'UA'}->request(GET $url);
    my $res = XML::Simple->new->XMLin($res_obj->content);

    print Dumper($res_obj);

}

sub add_overlay_file
{
    my ($self, $app_id, $filename, $path, $owner, $group, $permissions, $content) = @_;

    my $url = URI->new($self->{'API'} . "/api/v2/user/files");
    $url->query_form(appliance_id => $app_id,
                     filename => $filename,
                     path => $path,
                     owner => $owner,
                     group => $group,
                     permissions => $permissions); 
    
    my $res_obj = $self->{'UA'}->request(POST $url, 
                                         Content_Type => 'form-data',
                                         Content => [ file => [ undef, $filename, Content => $content ]]);
#    my $res = XML::Simple->new->XMLin($res_obj->content);
}

sub add_overlay_file_local
{
    my ($self, $app_id, $filename, $path, $owner, $group, $permissions, $file) = @_;
    
    my $url = URI->new($self->{'API'} . "/api/v2/user/files");
    $url->query_form(appliance_id => $app_id,
                     filename => $filename,
                     path => $path,
                     owner => $owner,
                     group => $group,
                     permissions => $permissions); 
    
    my $res_obj = $self->{'UA'}->request(POST $url,
                                         Content_Type => 'form-data',
                                         Content => [ file => [ $file ]]);
#    my $res = XML::Simple->new->XMLin($res_obj->content);
}

sub export_kiwi_configuration
{
    my ($self, $app_id, $type) = @_;
    
    my $req = POST($self->{'API'} . "/api/v2/user/kiwi_export?appliance_id=$app_id&image_type=$type");
    my $res_obj = $self->{'UA'}->request($req);
    my $res = XML::Simple->new->XMLin($res_obj->content);
    my $running_build_id = $res->{'running_build'}{'id'};
#    print Dumper($res);

    my $try;
    my $build_id;
    
    for ($try = 0; $try < 120; $try ++) {
        $res_obj = $self->{'UA'}->request(GET $self->{'API'} . "/api/v2/user/kiwi_export?appliance_id=$app_id");
        $res = XML::Simple->new->XMLin($res_obj->content);
#        print Dumper($res);
        $build_id = $res->{'build'}{'id'};
        last if defined $build_id;
        sleep(1);
    }
    
    return unless defined $build_id && $build_id == $running_build_id;
    
    $res_obj = $self->{'UA'}->request(GET $self->{'API'} . "/api/v2/user/download_kiwi_export?build_id=$build_id");
    my($filename) = ($res_obj->header('Content-Disposition') =~ m/"([^"]+)"/); 

    return unless $filename;

    open FILE, ">$filename" or die "Couldn't open file: $!"; 
    print FILE $res_obj->content; 
    close FILE;
    
    return $filename;
}   
