#!/usr/bin/perl -w

#
# Generates the DNS configuration and zonedata from LDAP
#


use Getopt::Long;
use POS::LDAP;
use Net::DNS;
use Net::IP;
use Net::IPv4Addr;
use Socket;
use File::Path;
use File::Copy;
use File::Compare;
use IO::File;
use POSIX qw(tmpnam);
use Time::localtime;
use URI::Escape;
use POS::Config;
use POS::Syslog;
use POS::Pid;
use POS::LdapUtil;
use POS::posDns;
use POS::posDataStatic;
use POS::posDataCache;

use strict;


# version  of the package

my $VERSION = 0.1;


######################
# main program
######################

log_parse_argv();

# global variables

my %POS_GLOBALS;

#init static and dynamic cache here
my $static_obj = POS::posDataStatic->new();
if (!defined $static_obj) {    #error already processed in the module
    exit(1);
}

my $cache_obj = POS::posDataCache->new('STATIC' => $static_obj);
if (!defined $cache_obj) {     #error already processed in the module
    exit(1);
}

$POS_GLOBALS{'STATIC'} = $static_obj;
$POS_GLOBALS{'CACHE'}  = $cache_obj;


# parse command-line options
# config file options can be overwritten by command line options
if (!GetOptions(
        'ldaphost=s'     => \$POS_GLOBALS{'Ldap_Host'},
        'ldapbase=s'     => \$POS_GLOBALS{'Ldap_Base'},
        'zonefilepath=s' => \$static_obj->{'CONFIG'}{'POS_LDAP2DNS_ZONEFILEPATH'},
        'zoneconfname=s' => \$static_obj->{'CONFIG'}{'POS_LDAP2DNS_ZONECONFNAME'},
        'zonetemplate=s' => \$static_obj->{'CONFIG'}{'POS_LDAP2DNS_ZONETEMPLATE'},
    ) ||
    $#ARGV >= 0
  ) {
    log_alert_print('called with invalid parameters');
    print_usage();
    exit 1;
}

# check command-line/config-file options
# must be changed when config file is used
if (!defined $static_obj->{'CONFIG'}{'POS_LDAP2DNS_ZONEFILEPATH'}) {
    print_usage();
    exit 1;
}

# check if already running
$SIG{'INT'} = 'break';
my $pid_obj = POS::Pid->new(DIRECTORY => '/var/run/', TIMEOUT => 5);
if ($pid_obj->{'ERROR'}) {
    log_alert_print('already running');
    printf("$pid_obj->{'ERROR_MSG'}\n");
    exit(1);
}


my $dns = POS::posDns->new(%POS_GLOBALS);

exit $dns->ldap2dns();


sub print_usage
{
    print STDERR "usage:\n";
    print STDERR "  --ldaphost=<ldap hostname>\n";
    print STDERR "  --ldapbase=<ldap base>\n";
    print STDERR "  --zonefilepath=<path for zone files\n";
    print STDERR "  --zoneconfname=<path and name of include config file>\n";
    print STDERR "  --zonetemplate=<path and name of zone header template>\n";
    print STDERR "  --dnsmasterip=<IP address of master DNS server>\n";
    log_print_usage();
} ## end sub print_usage


END {
#delete only if pid assigned (no early ctrl+c), not forked to other or if pidfile is ours and not of some other instance already runing
    $pid_obj->delete() if defined $pid_obj && $$ == $pid_obj->get_pid() && $$ == $pid_obj->get_pidf();
} ## end END

sub break
{
    #main reason is to invoke END block in the ctrl+C evend to clear the pid file
    printf("\nctrl+C detected! \n");
    exit(1);
} ## end sub break
